@@ -11,9 +11,9 @@ from uuid import uuid4
 
 import pytest
 
-from virtualenv.discovery.builtin import Builtin, get_interpreter
+from virtualenv.discovery.builtin import Builtin, LazyPathDump, get_interpreter, get_paths
 from virtualenv.discovery.py_info import PythonInfo
-from virtualenv.info import fs_supports_symlink
+from virtualenv.info import IS_WIN, fs_supports_symlink
 
 
 @pytest.mark.skipif(not fs_supports_symlink(), reason="symlink not supported")
@@ -200,6 +200,40 @@ def test_returns_second_python_specified_when_more_than_one_is_specified_and_env
     assert result == mocker.sentinel.python_from_cli
 
 
+def test_discovery_absolute_path_with_try_first(tmp_path, session_app_data):
+    good_env = tmp_path / "good"
+    bad_env = tmp_path / "bad"
+
+    # Create two real virtual environments
+    subprocess.check_call([sys.executable, "-m", "virtualenv", str(good_env)])
+    subprocess.check_call([sys.executable, "-m", "virtualenv", str(bad_env)])
+
+    # On Windows, the executable is in Scripts/python.exe
+    scripts_dir = "Scripts" if IS_WIN else "bin"
+    exe_name = "python.exe" if IS_WIN else "python"
+    good_exe = good_env / scripts_dir / exe_name
+    bad_exe = bad_env / scripts_dir / exe_name
+
+    # The spec is an absolute path, this should be a hard requirement.
+    # The --try-first-with option should be rejected as it does not match the spec.
+    interpreter = get_interpreter(
+        str(good_exe),
+        try_first_with=[str(bad_exe)],
+        app_data=session_app_data,
+    )
+
+    assert interpreter is not None
+    assert Path(interpreter.executable) == good_exe
+
+
+def test_discovery_via_path_with_file(tmp_path, monkeypatch):
+    a_file = tmp_path / "a_file"
+    a_file.touch()
+    monkeypatch.setenv("PATH", str(a_file))
+    interpreter = get_interpreter(uuid4().hex, [])
+    assert interpreter is None
+
+
 def test_absolute_path_does_not_exist(tmp_path):
     """
     Test that virtualenv does not fail when an absolute path that does not exist is provided.
@@ -223,6 +257,7 @@ def test_absolute_path_does_not_exist(tmp_path):
         capture_output=True,
         text=True,
         check=False,
+        encoding="utf-8",
     )
 
     # Check that the command was successful
@@ -249,12 +284,31 @@ def test_absolute_path_does_not_exist_fails(tmp_path):
         capture_output=True,
         text=True,
         check=False,
+        encoding="utf-8",
     )
 
     # Check that the command failed
     assert process.returncode != 0, process.stderr
 
 
+def test_get_paths_no_path_env(monkeypatch):
+    monkeypatch.delenv("PATH", raising=False)
+    paths = list(get_paths({}))
+    assert paths
+
+
+def test_lazy_path_dump_debug(monkeypatch, tmp_path):
+    monkeypatch.setenv("_VIRTUALENV_DEBUG", "1")
+    a_dir = tmp_path
+    executable_file = "a_file.exe" if IS_WIN else "a_file"
+    (a_dir / executable_file).touch(mode=0o755)
+    (a_dir / "b_file").touch(mode=0o644)
+    dumper = LazyPathDump(0, a_dir, os.environ)
+    output = repr(dumper)
+    assert executable_file in output
+    assert "b_file" not in output
+
+
 @pytest.mark.usefixtures("mock_get_interpreter")
 def test_returns_first_python_specified_when_no_env_var_is_specified(mocker, monkeypatch, session_app_data):
     monkeypatch.delenv("VIRTUALENV_PYTHON", raising=False)
